@@ -45,18 +45,10 @@ static const KeyBinding bindings_motions[] = {
 	{ ",",                  ACTION(TOTILL_REVERSE)                      },
 	{ "+",                  ALIAS("j^")                                 },
 	{ "-",                  ALIAS("k^")                                 },
-	{ " ",                  ALIAS("<Space>")                            },
-	{ "<Space>",            ALIAS("l")                                  },
-	{ "<Backspace>",        ALIAS("h")                                  },
 	{ "B",                  ACTION(CURSOR_LONGWORD_START_PREV)          },
 	{ "b",                  ACTION(CURSOR_WORD_START_PREV)              },
-	{ "<C-h>",              ALIAS("<Backspace>")                        },
-	{ "<C-j>",              ALIAS("j")                                  },
-	{ "<C-n>",              ALIAS("j")                                  },
-	{ "<C-p>",              ALIAS("k")                                  },
 	{ "E",                  ACTION(CURSOR_LONGWORD_END_NEXT)            },
 	{ "e",                  ACTION(CURSOR_WORD_END_NEXT)                },
-	{ "<Enter>",            ALIAS("j")                                  },
 	{ "F",                  ACTION(TO_LEFT)                             },
 	{ "f",                  ACTION(TO_RIGHT)                            },
 	{ "go",                 ACTION(CURSOR_BYTE)                         },
@@ -104,7 +96,6 @@ static const KeyBinding bindings_textobjects[] = {
 	{ "a}",                 ALIAS("a{")                                 },
 	{ "ab",                 ALIAS("a(")                                 },
 	{ "aB",                 ALIAS("a{")                                 },
-	{ "ae",                 ACTION(TEXT_OBJECT_ENTIRE_OUTER)            },
 	{ "al",                 ACTION(TEXT_OBJECT_LINE_OUTER)              },
 	{ "ap",                 ACTION(TEXT_OBJECT_PARAGRAPH_OUTER)         },
 	{ "as",                 ACTION(TEXT_OBJECT_SENTENCE)                },
@@ -126,7 +117,6 @@ static const KeyBinding bindings_textobjects[] = {
 	{ "i}",                 ALIAS("i{")                                 },
 	{ "ib",                 ALIAS("i(")                                 },
 	{ "iB",                 ALIAS("i{")                                 },
-	{ "ie",                 ACTION(TEXT_OBJECT_ENTIRE_INNER)            },
 	{ "il",                 ACTION(TEXT_OBJECT_LINE_INNER)              },
 	{ "ip",                 ACTION(TEXT_OBJECT_PARAGRAPH)               },
 	{ "is",                 ACTION(TEXT_OBJECT_SENTENCE)                },
@@ -141,16 +131,8 @@ static const KeyBinding bindings_selections[] = {
 	{ "M",                  ACTION(SELECTIONS_RESTORE)                  },
 	{ "|",                  ACTION(SELECTIONS_UNION)                    },
 	{ "&",                  ACTION(SELECTIONS_INTERSECT)                },
-	{ "!",                  ACTION(SELECTIONS_COMPLEMENT)               },
+	{ "~",                  ACTION(SELECTIONS_COMPLEMENT)               },
 	{ "\\",                 ACTION(SELECTIONS_MINUS)                    },
-	{ "z|",                 ACTION(SELECTIONS_COMBINE_UNION)            },
-	{ "z&",                 ACTION(SELECTIONS_COMBINE_INTERSECT)        },
-	{ "z+",                 ACTION(SELECTIONS_COMBINE_LONGER)           },
-	{ "z-",                 ACTION(SELECTIONS_COMBINE_SHORTER)          },
-	{ "z<",                 ACTION(SELECTIONS_COMBINE_LEFTMOST)         },
-	{ "z>",                 ACTION(SELECTIONS_COMBINE_RIGHTMOST)        },
-//	{ "-",                  ACTION(SELECTIONS_ROTATE_LEFT)              },
-//	{ "+",                  ACTION(SELECTIONS_ROTATE_RIGHT)             },
 	{ "_",                  ACTION(SELECTIONS_TRIM)                     },
 	{ "<S-Tab>",            ACTION(SELECTIONS_ALIGN_INDENT_RIGHT)       },
 	{ "<Tab>",              ACTION(SELECTIONS_ALIGN_INDENT_LEFT)        },
@@ -171,7 +153,6 @@ static const KeyBinding bindings_operators[] = {
 	{ "7",                  ACTION(COUNT)                               },
 	{ "8",                  ACTION(COUNT)                               },
 	{ "9",                  ACTION(COUNT)                               },
-	{ "~",                  ALIAS("g~")                                 },
 	{ "=",                  ALIAS(":|fmt<Enter>")                       },
 	{ "<",                  ACTION(OPERATOR_SHIFT_LEFT)                 },
 	{ ">",                  ACTION(OPERATOR_SHIFT_RIGHT)                },
@@ -203,17 +184,11 @@ static const KeyBinding bindings_normal[] = {
 	{ "<C-j>",              ACTION(SELECTIONS_NEW_LINE_BELOW)           },
 	{ "<C-k>",              ACTION(SELECTIONS_NEW_LINE_ABOVE)           },
 	{ "<C-l>",              ACTION(SELECTIONS_REMOVE_COLUMN_EXCEPT)     },
-	{ "<C-n>",              ACTION(SELECTIONS_MATCH_WORD)               },
+	{ "<C-n>",              ALIAS("viw")                                },
 	{ "<C-p>",              ACTION(SELECTIONS_REMOVE_LAST)              },
 	{ "<C-r>",              ACTION(REDO)                                },
 	{ "<C-u>",              ACTION(SELECTIONS_PREV)                     },
-	{ "<C-w><Backspace>",   ALIAS("<C-w>k")                             },
 	{ "<C-w>c",             ALIAS(":q<Enter>")                          },
-	{ "<C-w><C-h>",         ALIAS("<C-w>k")                             },
-	{ "<C-w><C-j>",         ALIAS("<C-w>j")                             },
-	{ "<C-w><C-k>",         ALIAS("<C-w>k")                             },
-	{ "<C-w><C-l>",         ALIAS("<C-w>j")                             },
-	{ "<C-w><C-w>",         ALIAS("<C-w>j")                             },
 	{ "<C-w>h",             ALIAS("<C-w>k")                             },
 	{ "<C-w>j",             ACTION(WINDOW_NEXT)                         },
 	{ "<C-w>k",             ACTION(WINDOW_PREV)                         },
@@ -221,10 +196,8 @@ static const KeyBinding bindings_normal[] = {
 	{ "<C-w>n",             ALIAS(":open<Enter>")                       },
 	{ "<C-w>s",             ALIAS(":split<Enter>")                      },
 	{ "<C-w>v",             ALIAS(":vsplit<Enter>")                     },
-	{ "<C-w>w",             ALIAS("<C-w>j")                             },
 	{ "<C-y>",              ACTION(WINDOW_SLIDE_DOWN)                   },
 	{ "D",                  ALIAS("d$")                                 },
-	{ "<Delete>",           ALIAS("x")                                  },
 	{ "<Escape>",           ACTION(MODE_NORMAL_ESCAPE)                  },
 	{ "<F1>",               ALIAS(":help<Enter>")                       },
 	{ "ga",                 ACTION(UNICODE_INFO)                        },
@@ -270,12 +243,11 @@ static const KeyBinding bindings_visual[] = {
 	{ "+",                  ACTION(SELECTIONS_ROTATE_RIGHT)             },
 	{ "<",                  ALIAS("<vis-operator-shift-left>gv")        },
 	{ ">",                  ALIAS("<vis-operator-shift-right>gv")       },
-	{ "<Backspace>",        ALIAS("d")                                  },
+	{ "<C-a>",              ACTION(SELECTIONS_NEW_MATCH_ALL)            },
 	{ "<C-b>",              ALIAS("<PageUp>")                           },
 	{ "<C-c>",              ACTION(SELECTIONS_REMOVE_COLUMN)            },
 	{ "<C-d>",              ACTION(SELECTIONS_NEXT)                     },
 	{ "<C-f>",              ALIAS("<PageDown>")                         },
-	{ "<C-h>",              ALIAS("<Backspace>")                        },
 	{ "<C-j>",              ALIAS("<C-d>")                              },
 	{ "<C-k>",              ALIAS("<C-u>")                              },
 	{ "<C-l>",              ACTION(SELECTIONS_REMOVE_COLUMN_EXCEPT)     },
@@ -283,7 +255,6 @@ static const KeyBinding bindings_visual[] = {
 	{ "<C-p>",              ACTION(SELECTIONS_REMOVE_LAST)              },
 	{ "<C-u>",              ACTION(SELECTIONS_PREV)                     },
 	{ "<C-x>",              ACTION(SELECTIONS_NEW_MATCH_SKIP)           },
-	{ "<Delete>",           ALIAS("<Backspace>")                        },
 	{ "<Escape>",           ACTION(MODE_VISUAL_ESCAPE)                  },
 	{ "I",                  ACTION(SELECTIONS_NEW_LINES_BEGIN)          },
 	{ "J",                  ACTION(JOIN_LINES)                          },
